/* Copyright (c) 2009 PrimeBase Technologies GmbH, Germany
 *
 * PrimeBase Media Stream for MySQL
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * Barry Leslie
 *
 * 2009-05-29
 *
 * H&G2JCtL
 *
 * Repository backup.
 *
 */

#ifndef _BACKUP_MS_H_
#define _BACKUP_MS_H_
class MSDatabase;

class MSBackupInfo : public CSRefObject {
	private:
	static csWord4	gMaxInfoRef;
	static CSSyncSparseArray *gBackupInfo;

	friend class MSBackupTable;
	friend class MSBackup;
	
	private:	
	csWord4			backupRefId;
	CSString		*db_name;
	csWord4			db_id;
	time_t			startTime;
	time_t			completionTime;
	bool			dump;
	bool			isRunning;
	CSString		*backupLocation;
	csWord4			cloudRef;
	csWord4			cloudBackupNo;
	
public:

	static void startUp()
	{
		new_(gBackupInfo, CSSyncSparseArray(5));
		gMaxInfoRef = 0;
	}
	
	static void shutDown()
	{
		if (gBackupInfo) {
			gBackupInfo->clear();
			gBackupInfo->release();
			gBackupInfo = NULL;
		}	
	}


	static MSBackupInfo *findBackupInfo(csWord4 backupRefId)
	{
		MSBackupInfo *info;
		enter_();
		
		lock_(gBackupInfo);
		
		info = (MSBackupInfo *) gBackupInfo->get(backupRefId);
		if (info) 
			info->retain();
		unlock_(gBackupInfo);
		return_(info);
	}
	
	static MSBackupInfo *getBackupInfo(csWord4 backupRefId)
	{
		MSBackupInfo *info = findBackupInfo(backupRefId);
		if (!info) {
			enter_();
			char msg[80];
			snprintf(msg, 80, "Backup info with reference ID %u not found", backupRefId);
			CSException::throwException(CS_CONTEXT, CS_ERR_GENERIC_ERROR, msg);
			outer_();
		}
		return info;
	}
	
	
	MSBackupInfo(csWord4 id, const char *name, csWord4 db_id, time_t start, time_t end, bool isDump, const char *location, csWord4 cloudRef_arg, csWord4 cloudBackupNo_arg );
	~MSBackupInfo();
	
	csWord4 getBackupRefId() { return backupRefId;}
	
	const char *getName(){ return db_name->getCString(); }
	
	csWord4 getDatabaseId() { return db_id;}
	
	time_t getStart(){ return startTime;}
	
	time_t getEnd(){ return completionTime;}
	
	bool isDump(){ return dump;}
	
	bool isBackupRunning(){ return isRunning;}

	const char *getLocation() { return (backupLocation)?backupLocation->getCString():NULL; }
		
	csWord4 getcloudRef(){ return cloudRef;}
	void setcloudRef(csWord4 no){ cloudRef = no;}

	csWord4 getcloudBackupNo(){ return cloudBackupNo;}
	void setcloudBackupNo(csWord4 no){ cloudBackupNo = no;}
	
	static MSBackupInfo *startDump(MSDatabase *db, csWord4 cloud_ref, csWord4 backup_no);

	void startBackup(MSDatabase *pbms_db);
	void backupCompleted(MSDatabase *db);
	void backupTerminated(MSDatabase *db);
};


class MSDatabase;
class MSOpenSystemTable;

class MSBackup :public CSDaemon {

public:

	MSBackup();
	~MSBackup();
	
	virtual bool doWork();

	virtual void *finalize();
	
	void startBackup(MSDatabase *src_db);
	csWord8 getBackupSize() { return bu_size;}
	csWord8 getBackupCompletedSize() { return bu_completed;}
	bool	isRunning() { return bu_BackupRunning;}
	int		getStatus() { return (bu_BackupRunning)?0:bu_State;}
	csWord4 backupID() { return bu_ID;}
	
	static MSBackup* newMSBackup(MSBackupInfo *backup_info);

private:
	void completeBackup();
	
	MSBackupInfo *bu_info;

	CSVector	*bu_BackupList;
	CSDaemon	*bu_Compactor;
	bool		bu_BackupRunning;
	enum		{BU_RUNNING = -1, BU_COMPLETED = 0, BU_TERMINATED = 1}	bu_State; 
	
	MSDatabase	*bu_SourceDatabase;	// The source database.
	MSDatabase	*bu_Database;		// The destination database.
	MSOpenSystemTable *bu_dst_dump;	// The source database's pbms_dump.
	MSOpenSystemTable *bu_src_dump;	// The source database's pbms_dump.
	csWord8		bu_size;			// The total size of the data to be backed up.
	csWord8		bu_completed;		// The amount of data that has been backed up so far.
	
	csWord4		bu_ID;
	csWord4		bu_start_time;
	
	bool		bu_TransactionManagerSuspended;
};

#endif // _BACKUP_MS_H_
